##
# $Id: options.rb 9550 2010-06-17 14:03:53Z todb $
##

##
# This file is part of the Metasploit Framework and may be subject to
# redistribution and commercial restrictions. Please see the Metasploit
# Framework web site for more information on licensing and terms of use.
# http://metasploit.com/framework/
##

require 'msf/core'
require 'savon'
class Metasploit3 < Msf::Auxiliary

	# Exploit mixins should be called first
	include Msf::Exploit::Remote::HttpClient
	include Msf::Auxiliary::WMAPScanServer
	# Scanner mixin should be near last
	include Msf::Auxiliary::Scanner
	include Msf::Auxiliary::Report

	def initialize
		super(
			'Name'        => 'SOAP Basic Request',
			'Version'     => '$Revision: 9550 $',
			'Description' => 'This module makes a basic SOAP request based w/o using a WSDL.',
			'Author'       => ['jabra[at]rapid7.com'],
			'License'     => MSF_LICENSE
		)
         register_options(
            [
                OptString.new('NAMESPACE', [ true,  "The namespace location", '']),
                OptString.new('SCHEME', [ true,  "The scheme to use.", 'http']),
                OptString.new('ACTION', [ true,  "The action to use.", '']),
                OptString.new('PARAM', [ true,  "The parameter to use.", '']),
                OptString.new('VALUE', [ true,  "The value to use.", '']),
                OptString.new('PROXYIP', [ true,  "The IP of the webproxy to use", 'localhost']),
                OptString.new('PROXYPORT', [ true,  "The port of the webproxy to use", '8080']),
                OptString.new('PROXYENABLE', [ true,  "Enable/Disable the webproxy", 'false']),
                OptString.new('USERNAME', [ true,  "The username to use for WSSE authentication", 'root']),
                OptString.new('PASSWORD', [ true,  "The password to use for WSSE authentication", 'toor']),
                OptString.new('AUTH', [ true,  "The type of WSSE authentication (BASIC or DIGEST)", 'NONE']),
            ], self.class)

	end

	def run_host(target_host)
		begin
            scheme = datastore['SCHEME'] 
            path = datastore['PATH'] 
            port = datastore['RPORT'] 
            action = datastore['ACTION'] 
            param = datastore['PARAM'] 
            value = datastore['VALUE'] 
            namespace = datastore['NAMESPACE']

            if (datastore['PROXYENABLE'] == 'true')
                proxyip = datastore['PROXYIP']
                proxyport = datastore['PROXYPORT']
                client = Savon::Client.new "#{scheme}://#{target_host}:#{port}/#{path}", :proxy => "http://#{proxyip}:#{proxyport}"
            else
                client = Savon::Client.new "#{scheme}://#{target_host}:#{port}/#{path}"
            end
            
            if (datastore['AUTH'].upcase == 'digest' or datastore['AUTH'].upcase == 'basic')
                Savon::WSSE.username = datastore['USERNAME']
                Savon::WSSE.password = datastore['PASSWORD']
                if (datastore['AUTH'].upcase == 'digest')
                    Savon::WSSE.digest = true
                else
                    Savon::WSSE.digest = false
                end
            end

		    # Savon::Request.log = false
            # Executing a request to call a SOAP action.
            response = client.send(action.to_s + "!") do |soap|
                soap.namespace = namespace
                # camelize action
                soap.action = action.split('_').map {|w| w.capitalize}.join
                soap.input =  "#{soap.action}Request"
                soap.body = { "wsdl:#{param}" => "#{value}" }
            end
            print_status(response.to_yaml)
            
        rescue ::Rex::ConnectionRefused, ::Rex::HostUnreachable, ::Rex::ConnectionTimeout
            rescue ::Timeout::Error, ::Errno::EPIPE
		end
	end
end

