##
# $Id: options.rb 9550 2010-06-17 14:03:53Z todb $
##

##
# This file is part of the Metasploit Framework and may be subject to
# redistribution and commercial restrictions. Please see the Metasploit
# Framework web site for more information on licensing and terms of use.
# http://metasploit.com/framework/
##

require 'msf/core'
require 'savon'
class Metasploit3 < Msf::Auxiliary

	# Exploit mixins should be called first
	include Msf::Exploit::Remote::HttpClient
	include Msf::Auxiliary::WMAPScanServer
	# Scanner mixin should be near last
	include Msf::Auxiliary::Scanner
	include Msf::Auxiliary::Report

	def initialize
		super(
			'Name'        => 'SOAP Enumeration',
			'Version'     => '$Revision: 9550 $',
			'Description' => 'This module uses a WSDL to enumerate the actions of a SOAP Web Service.',
			'Author'       => ['jabra[at]rapid7.com'],
			'License'     => MSF_LICENSE
		)
         register_options(
            [
                OptString.new('PATH', [ true,  "The path to the WSDL", '/']),
                OptString.new('SCHEME', [ true,  "The scheme use ", 'http']),
                OptString.new('PROXYIP', [ true,  "The IP of the webproxy to use", 'localhost']),
                OptString.new('PROXYPORT', [ true,  "The port of the webproxy to use", '8080']),
                OptString.new('PROXYENABLE', [ true,  "Enable/Disable the webproxy", 'false']),
            ], self.class)
	end

	def run_host(target_host)
		begin
            scheme = datastore['SCHEME'] 
            path = datastore['PATH'] 
            port = datastore['RPORT'] 
            if (datastore['PROXYENABLE'] == 'true') 
                proxyip = datastore['PROXYIP']
                proxyport = datastore['PROXYPORT']
                client = Savon::Client.new "#{scheme}://#{target_host}:#{port}/#{path}", :proxy => "http://#{proxyip}:#{proxyport}"
            else
                client = Savon::Client.new "#{scheme}://#{target_host}:#{port}/#{path}"
            end

		    Savon::Request.log = false
		    print_status("Enumerating #{scheme}://#{target_host}:#{port}/#{path}")
		    print_status("Namespace: #{client.wsdl.namespace_uri}")
            print_status("Endpoint: #{client.wsdl.soap_endpoint}")
		    print_status("SOAP Actions:")
            client.wsdl.soap_actions.each do |s|
                print_status(" - #{s}")
            end
            rescue ::Rex::ConnectionRefused, ::Rex::HostUnreachable, ::Rex::ConnectionTimeout
            rescue ::Timeout::Error, ::Errno::EPIPE
		end
	end
end

